#! /usr/bin/python

import sys, os, glob
import numpy as np
import matplotlib.pylab as plt

#=================================================================================================================
def read_gyre_in(filename, h5=False):
  """
  Read input model to GYRE in ASCII or HDF5 (GSM) format.
  @param filename: full path to the input GYRE model
  @type filename: string
  @param h5: whether or not the input file is ascii (False) or HDF5 (GSM, True); default=False
  @type h5: boolean
  @return: dictionary with these keys and values:
   - nn: number of mesh
   - M_star: total mass in gr 
   - R_star: total radius in cm 
   - L_star: total luminosity in cgs
   - ncol: number of data columns; default=19
   - gyre_in: numpy record array giving the data
  @rtype: dictionary   
  """
  if not os.path.exists(filename):
    message = 'Error: read: read_gyre_in: {0} does not exist'.format(filename)
    raise SystemExit, message
  
  if h5:
    message = 'Error: read: read_gyre_in: reading GSM/HDF5 is not supported yet'
    raise SystemExit, message
  else:
    dic = {}
    col_names = 'zone, radius, q_div_xq, luminosity, pressure, temperature, density, gradT, brunt_N2, gamma1, grada,' \
                 'chiT_div_chiRho, kappa, kappa_T, kappa_rho, epsilon, epsilon_T, epsilon_rho, omega'

    with open(filename, 'r') as f: lines = f.readlines()
    header = lines[0].rstrip('\r\n').split()
    nn     = int(header[0])
    M_star = float(header[1])
    R_star = float(header[2])
    L_star = float(header[3])
    ncol   = int(header[4])
    
    col_dtype = 'i8,' + 'f8,' * (ncol - 1)

    data   = []
    for line in lines[1:]: 
      splitted = line.rstrip('\r\n').split()
      float_line = []
      float_line.append(int(splitted[0]))
      for i in range(1, ncol): float_line.append(float(splitted[i].replace('D', 'e')))
      data.append(float_line)
    
    recarr = np.core.records.fromarrays(np.array(data).transpose(), names=col_names, formats=col_dtype[:-1])
    
    dic['nn']     = nn
    dic['M_star'] = M_star
    dic['R_star'] = R_star
    dic['L_star'] = L_star
    dic['ncol']   = ncol
    dic['gyre_in']= recarr
    
  return dic
    
#=================================================================================================================
def check_Hydr_Equil(dic, Lagr=True, Euler=False, file_out=None):
  """
  Plot and ensure that the input GYRE model is in hydrostatic equilibrium point by point
  @param dic: dictinary provided by mesa_gyre.read.read_gyre_in, containing data from the file. 
  @type dic: dictionary
  @param file_out: full path to the output plot
  @type file_out: string
  @param Lagr, Euler: to check the hydrostatic equilibrium in Lagrangian vs. Eulerian coordinate
  @type Lagr, Euler: boolean
  @return: None
  @rtype: None
  """
  if type(dic) is not type({}):
    message = 'Error: plot_gyre: check_Hydr_Equil: First argument is not a dictionary!'
    raise SystemExit, message
  
  c_grav = 6.67428e-8 # cgs; as defined in mesa/const/const_def.f
  
  nn     = dic['nn']
  M_star = dic['M_star']
  R_star = dic['R_star']
  L_star = dic['L_star']
  ncol   = dic['ncol']
  rec    = dic['gyre_in']
  
  r_cm   = rec['radius']
  P      = rec['pressure']
  Rho    = rec['density']
  logT   = np.log10(rec['temperature'])
  q_div_xq = rec['q_div_xq']
  w      = q_div_xq

  m_gr   = np.zeros(nn)
  for i in range(0, nn-1): m_gr[i] = M_star * w[i] / (1.0 + w[i])
  m_gr[nn-1] = M_star
  dm = np.zeros(nn)    
  dm[1:] = m_gr[1:] - m_gr[0:nn-1]
  dm[0]  = m_gr[0]
  #dm[0:nn-1] = m_gr[1:nn] - m_gr[0:nn-1]

  gravity= - c_grav * m_gr[:] / r_cm[:]**2.0
  
  dr = np.zeros(nn)
  dr[1:nn] = r_cm[1:nn] - r_cm[0:nn-1]
  dr[0] = r_cm[0]
  
  lhs = np.zeros(nn)
  rhs = np.zeros(nn)
  for i in range(1, nn):
    if Lagr:
      lhs[i] = (P[i-1] - P[i]) / ((dm[i-1] + dm[i]) / 2.0)
      rhs[i] = -c_grav * m_gr[i] / (4.0 * np.pi * r_cm[i]**4.0)
    if Euler:
      lhs[i] = (P[i-1] - P[i]) / ((dr[i-1] + dr[i]) / 2.0)
      rhs[i] = -Rho[i] * gravity[i]
  lhs[0] = lhs[1]
  rhs[0] = rhs[1]
  
  departure = np.abs(1.0 - lhs[:]/rhs[:])
  if Lagr: txt = 'Lagrangian'
  else: txt = 'Eulerian'
  print ' - plot_gyre: check_Hydr_Equil: ' + txt + ' frame'
  print '   Median of departure = %s' % (np.median(departure), )
  print '   Min = %s, Max= %s' % (min(np.abs(departure)), max(departure))

  if file_out:
    fig = plt.figure(figsize=(6,4), dpi=200)
    ax  = fig.add_subplot(111)
    plt.subplots_adjust(left=0.12, bottom=0.12)
    ax.set_xlim(logT.max(), logT.min())
    ax.set_ylim(-20, 1)
    ax.set_xlabel(r'log T')
    ax.set_ylabel(r'$\log Q = \log|1.0 - \frac{\rm LHS}{\rm RHS}|$')
    ax.plot(logT, np.log10(departure), lw=1, color='black', label=r'$\log Q$')
    ax.plot(logT, np.log10(np.abs(lhs)), lw=1, color='blue', label=r'$\log$(LHS)')
    ax.plot(logT, np.log10(np.abs(rhs)), lw=1, color='red', label=r'$\log$(RHS)')
    leg = ax.legend(loc=0, fontsize='x-small')    
    plt.savefig(file_out, dpi=200)
    print ' - plot_gyre: check_Hydr_Equil: %s saved' % (file_out, )
    plt.close()

  return None
  
#=================================================================================================================
def main():
  gyre_file = 'profile1.prof.GYRE'
  dic_gyre  = read_gyre_in(gyre_file)
  check_Hydr_Equil(dic_gyre, file_out='Hydr-Equil-GYRE.pdf')
  
  return 0

#=================================================================================================================
if __name__=='__main__':
  status = main()
  sys.exit(status)
#=================================================================================================================  